#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>


#ifdef USE_RT_SIGNAL
#define SIGNO (SIGRTMIN+0)		/* RT signal */
#else
#define SIGNO SIGUSR1		/* normal signal */
#endif

#define NOF_DELAY 10000

int delay[NOF_DELAY];
int idel;
struct timeval start;

void handle_signal(int signal)
{
  struct timeval end;
  int delta;

  gettimeofday(&end, NULL);
  delta = (end.tv_sec - start.tv_sec) * 1000000 +
    (end.tv_usec - start.tv_usec);

  delay[idel++] = delta;
}

void print_hist(void)
{
#define HISTSIZE 1000

  int hist[1000 + 1];
  int i;

  memset(hist, 0, sizeof(hist));

  for (i = 0; i < NOF_DELAY; i++) {
    if (delay[i] >= HISTSIZE) {
      printf("sample %d over max hist: %d\n", i, delay[i]);
      hist[HISTSIZE]++;
    }
    else {
      hist[delay[i]]++;
    }
  }
  for (i = 0; i < HISTSIZE; i++) {
    if (hist[i]) {
      printf("%d: %d\n", i, hist[i]);
    }
  }
  printf("HC: %d\n", hist[HISTSIZE]);
}

void print_avg(void)
{
  double sum;
  int i;

  sum = 0;
  for (i = 0; i < NOF_DELAY; i++) {
    sum += delay[i];
  }

  printf("avg. delay: %f\n", sum/NOF_DELAY);
}

int main(void)
{
  struct sigaction act;
  int i;
  pid_t p;
  sigset_t set;

  sigemptyset(&set);
  sigprocmask(SIG_SETMASK, &set, NULL);

  p = getpid();

  act.sa_flags = 0;
  act.sa_handler = handle_signal;
  sigemptyset(&act.sa_mask);

  if (sigaction(SIGNO, &act, NULL)) {
    perror("sigaction failed");
    exit(1);
  }

  for (i = 0; i < NOF_DELAY; i++) {
    gettimeofday(&start, NULL);
    kill(p, SIGNO);
  }

  assert(idel == NOF_DELAY);

  print_hist();

  print_avg();

  return 0;
}
